function [ConservedComplexes] = ClusterM(Network, O_Pairs, NameList, Lambda, NumCores)
%% Inputs:
% "Network" is a cell, each elemnet of which contains a sparse adjacency
% matrix of a PPI network
% "O_Pairs" is a cell, each elemnet of which contains similar protein pair
% with the protien sequence similariy score ([0,1]) between them.
% "NameList" is a cell, each element of which contains the number of the
% node in "Network"
% "NumCores" indicates the number of cores to use



%% Number of PPI networks
NumNet = length(Network);

%% add in self loops to all networks
for i = 1 : NumNet
    TempN = length(Network{i});
    Network{i} = Network{i}-diag(diag(Network{i}))+sparse(1:TempN,1:TempN,ones(1,TempN), TempN, TempN);
end

%% set the largest size of the subnetworks
% SizeSubNetwork can be changed form 20 to 100; 30 is default value
SizeSubNetwork = 30;
LocalNumNodes = cell(1, NumNet);
for i = 1 : NumNet
    LocalNumNodes{i} = SizeSubNetwork;
end

%% threshodl for approxmiating pagerank vecotr
% don't need to be changed
for i = 1 : NumNet
    CondT{i} = 0.5;
end

%% compute parameters for approxmiating pagerank vecotr
for i = 1 : NumNet
    [NetNumNodes{i}, NetDeg{i}, alpha{i}, eps{i}] = GetNetworkInfo(Network{i}, CondT{i});
    CoClusterUsed{i} = [];
end

%% for similarity
[Sim] = GetSimInfo(NetNumNodes, O_Pairs);


%% Step 1:
%% Find homologous protien spines
orth_nodes = SMETANA4ClusterM(Network, O_Pairs, NetNumNodes, NameList, ['SMETNA_Alignment_Result.txt']);

%% Step 2:
%% Detect conserved protien complexes
partition_co = cell(1,length(orth_nodes));
conductance = zeros(1,length(orth_nodes));
SimT = 0;
%parpool(NumCores);
matlabpool open
parfor i = 1 : length(orth_nodes(:,1))
    [RawComplexes{i}, conductance(i)] = LocalClusterM_Greedy(NetNumNodes, orth_nodes(i,:), Network, NetDeg, alpha, eps, LocalNumNodes, Sim, SimT, Lambda);
end
matlabpool close
%poolobj = gcp('nocreate');
%delete(poolobj);

%% Step 3:
%% Remove duplicated conserved complexes or complexes whose similairty score less than a threholds
[Compelxes] = PostProcessing(Network, RawComplexes, NameList);
    
    
    
    
    
